/* ------------------------------------------------------------------------------

	FILENAME
		JobFormatModeLibrary.c
		
	DESCRIPTION
		Handy job format mode functions.
		
	COPYRIGHT
		  Apple Computer, Inc. 1992
		 All rights reserved. 
	
-------------------------------------------------------------------------------- */


#include <PrintingManager.h>
#include <PrintingLibraries.h>

/* Wrappers for JobFormatModeQuery() */


/****************************************************************************************

							GetJobFormatLineConstraint
							
	function :
					Returns position constraint table for direct mode line drawing
	parameters :
					the job and optional handle to hold the position constraint table
					
****************************************************************************************/
gxPositionConstraintTableHdl GetJobFormatLineConstraint(gxJob theJob, gxPositionConstraintTableHdl hPositionConstraintTable) {
	
	GXJobFormatModeQuery(theJob, gxGetJobFormatLineConstraintQuery, NULL, &hPositionConstraintTable);
	
	return hPositionConstraintTable;
}


/****************************************************************************************

							GetJobFormatFonts
							
	function :
					Returns font table for direct mode
	parameters :
					the job and optional handle to hold the font table
					
****************************************************************************************/
gxFontTableHdl GetJobFormatFonts(gxJob theJob, gxFontTableHdl hFontTable) {
	
	GXJobFormatModeQuery(theJob, gxGetJobFormatFontsQuery, NULL, &hFontTable);
	
	return hFontTable;
}


/****************************************************************************************

							GetJobFormatFontCommonStyles
							
	function :
					Returns style name table for direct mode
	parameters :
					the job and optional handle to hold the style name table
					
****************************************************************************************/
gxStyleNameTableHdl GetJobFormatFontCommonStyles(gxJob theJob, gxFont theFont, gxStyleNameTableHdl hStyleTable) {
	
	GXJobFormatModeQuery(theJob, gxGetJobFormatFontCommonStylesQuery, theFont, &hStyleTable);
	
	return hStyleTable;
}


/****************************************************************************************

							GetJobFormatFontConstraint
							
	function :
					Returns position constraint table for direct mode fonts
	parameters :
					the job and optional handle to hold the position constraint table
					
****************************************************************************************/
gxPositionConstraintTableHdl GetJobFormatFontConstraint(gxJob theJob, gxFont theFont, gxPositionConstraintTableHdl hPositionConstraintTable) {
	
	GXJobFormatModeQuery(theJob, gxGetJobFormatFontConstraintQuery, theFont, &hPositionConstraintTable);
	
	return hPositionConstraintTable;
}


/****************************************************************************************

							SetStyleJobFormatCommonStyle
							
	function :
					Returns the style with the additional font style information set
	parameters :
					the job and a style with the font and size values set
					
****************************************************************************************/
gxStyle SetStyleJobFormatCommonStyle(gxJob theJob, Str255 theStyleName, gxStyle theStyle) {
	
	GXJobFormatModeQuery(theJob, gxSetStyleJobFormatCommonStyleQuery, theStyleName, theStyle);
	
	return theStyle;
}
